/*
_____       _    _    Corso   Italia,  178
(_|__   .  (_   |_|_  56125           Pisa
(_|_) |)|(()_)()| |   tel.  +39  050 46380
  |   |               picosoft@picosoft.it

 picoODBC - picoSQL ODBC driver

 Copyright (C) Picosoft s.r.l. 1994-2002

 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Lesser General Public
 License as published by the Free Software Foundation; either
 version 2.1 of the License, or (at your option) any later version.

 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public
 License along with this library; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
static char rcsid[] = "$Id: catalog.cpp 4.0 1998/10/27 16:19:54 picoSoft Exp $";
# include <sql.h>
# include <sqlext.h>
# ifdef WIN32
# include <windows.h>
# endif
# include <stdio.h>
# include "sqlhndls.h"
# include "dberror.h"
inline PBool
notOk (unsigned char * sz, SWORD cb) {
   return sz != 0 && cb < 0 && cb != SQL_NTS;
}
extern "C" {


//  Have DBMS set up result set of Columns.

RETCODE SQL_API SQLColumns(
    HSTMT     hstmt,
    UCHAR FAR *szTableQualifier,
    SWORD     cbTableQualifier,
    UCHAR FAR *szTableOwner,
    SWORD     cbTableOwner,
    UCHAR FAR *szTableName,
    SWORD     cbTableName,
    UCHAR FAR *szColumnName,
    SWORD     cbColumnName)
{ 
   CORBA_Long Return = SQL_SUCCESS;
   ClntHstmt *myHstmt = (ClntHstmt *) hstmt;
   void *argv[10];

   if (!myHstmt || !myHstmt->IsA(ClntHstmt::Class))
      return SQL_INVALID_HANDLE;
   else if (notOk (szTableQualifier, cbTableQualifier) ||
            notOk (szTableOwner,     cbTableOwner) || 
            notOk (szTableName,      cbTableName) || 
            notOk (szColumnName,     cbColumnName)) {
      AddError(0, 0, myHstmt, "S1090", 0, "Invalid string or buffer length");
      return SQL_ERROR;
   }
   DelError(0,0,myHstmt);

   argv[0] = &Return;
   argv[1] = &myHstmt->srvHstmt;
   if (cbTableQualifier > 0) {
      argv[2] = new char[cbTableQualifier + 1];
      memcpy (argv[2], szTableQualifier, cbTableQualifier);
      ((char **) argv)[2][cbTableQualifier] = '\0';
   } else
     argv[2] = szTableQualifier;
   argv[3] = &cbTableQualifier;
   if (cbTableOwner > 0) {
      argv[4] = new char[cbTableOwner + 1];
      memcpy (argv[4], szTableOwner, cbTableOwner);
      ((char **) argv)[4][cbTableOwner] = '\0';
   } else
      argv[4] = szTableOwner;
   argv[5] = &cbTableOwner;
   if (cbTableName > 0) {
      argv[6] = new char[cbTableName + 1];
      memcpy (argv[6], szTableName, cbTableName);
      ((char **) argv)[6][cbTableName] = '\0';
   } else
      argv[6] = szTableName;
   argv[7] = &cbTableName;
   if (cbColumnName > 0) {
      argv[8] = new char[cbColumnName + 1];
      memcpy (argv[8], szColumnName, cbColumnName);
      ((char **) argv)[8][cbColumnName] = '\0';
   } else
      argv[8] = szColumnName;
   argv[9] = &cbColumnName;
   
   Exception *ex = Exception_new();
   if (!Invoke_invoke (myHstmt->hdbc->call,ex,"SQLColumns",argv)) {
      char stat[6];
      sprintf (stat,"08%03d", ex->code);
      AddError (0, 0, myHstmt, stat, ex->code, ex->description);
      Exception_log(ex);
   }
   if (argv[2] != szTableQualifier)
      delete (char*) argv[2];
   if (argv[4] != szTableOwner)
      delete (char*) argv[4];
   if (argv[6] != szTableName)
      delete (char*) argv[6];
   if (argv[8] != szColumnName)
      delete (char*) argv[8];
   Exception_delete (ex);
   return (RETCODE) Return;
}


//  Have DBMS set up result set of Tables.

RETCODE SQL_API SQLTables(
    HSTMT     hstmt,
    UCHAR FAR *szTableQualifier,
    SWORD     cbTableQualifier,
    UCHAR FAR *szTableOwner,
    SWORD     cbTableOwner,
    UCHAR FAR *szTableName,
    SWORD     cbTableName,
    UCHAR FAR *szTableType,
    SWORD     cbTableType)
{ 
   CORBA_Long Return = SQL_ERROR;
   ClntHstmt *myHstmt = (ClntHstmt *) hstmt;
   void *argv[10];

   if (!myHstmt || !myHstmt->IsA(ClntHstmt::Class))
      return SQL_INVALID_HANDLE;
   else if (notOk (szTableQualifier, cbTableQualifier) ||
            notOk (szTableOwner,     cbTableOwner) || 
            notOk (szTableName,      cbTableName) || 
            notOk (szTableType,      cbTableType)) {
      AddError(0, 0, myHstmt, "S1090", 0, "Invalid string or buffer length");
      return SQL_ERROR;
   }
   DelError(0,0,myHstmt);

   argv[0] = &Return;
   argv[1] = &myHstmt->srvHstmt;
   if (cbTableQualifier > 0) {
      argv[2] = new char[cbTableQualifier + 1];
      memcpy (argv[2], szTableQualifier, cbTableQualifier);
      ((char **) argv)[2][cbTableQualifier] = '\0';
   } else
      argv[2] = szTableQualifier;
   argv[3] = &cbTableQualifier;
   if (cbTableOwner > 0) {
      argv[4] = new char[cbTableOwner + 1];
      memcpy (argv[4], szTableOwner, cbTableOwner);
      ((char **) argv)[4][cbTableOwner] = '\0';
   } else
      argv[4] = szTableOwner;
   argv[5] = &cbTableOwner;
   if (cbTableName > 0) {
      argv[6] = new char[cbTableName + 1];
      memcpy (argv[6], szTableName, cbTableName);
      ((char **) argv)[6][cbTableName] = '\0';
   } else
      argv[6] = szTableName;
   argv[7] = &cbTableName;
   if (cbTableType > 0) {
      argv[8] = new char[cbTableType + 1];
      memcpy (argv[8], szTableType, cbTableType);
      ((char **) argv)[8][cbTableType] = '\0';
   } else
      argv[8] = szTableType;
   argv[9] = &cbTableType;
   
   Exception *ex = Exception_new();
   if (!Invoke_invoke (myHstmt->hdbc->call,ex,"SQLTables",argv)) {
      char stat[6];
      sprintf (stat,"08%03d", ex->code);
      AddError (0, 0, myHstmt, stat, ex->code, ex->description);
      Exception_log(ex);
   }
   if (argv[2] != szTableQualifier)
      delete (char*) argv[2];
   if (argv[4] != szTableOwner)
      delete (char*) argv[4];
   if (argv[6] != szTableName)
      delete (char*) argv[6];
   if (argv[8] != szTableType)
      delete (char*) argv[8];
   Exception_delete (ex);
   return (RETCODE) Return;
}


//  Have DBMS set up result set of Statistics.


RETCODE SQL_API SQLStatistics(
    HSTMT     hstmt,
    UCHAR FAR *szTableQualifier,
    SWORD     cbTableQualifier,
    UCHAR FAR *szTableOwner,
    SWORD     cbTableOwner,
    UCHAR FAR *szTableName,
    SWORD     cbTableName,
    UWORD     fUnique,
    UWORD     fAccuracy)    
{
   CORBA_Long Return = SQL_ERROR;
   ClntHstmt *myHstmt = (ClntHstmt *) hstmt;
   void *argv[10];

   if (!myHstmt || !myHstmt->IsA(ClntHstmt::Class))
      return SQL_INVALID_HANDLE;
   else if (notOk (szTableQualifier, cbTableQualifier) ||
            notOk (szTableOwner,     cbTableOwner) || 
            notOk (szTableName,      cbTableName)) {
      AddError(0, 0, myHstmt, "S1090", 0, "Invalid string or buffer length");
      Return = SQL_ERROR;
   } else if (fUnique != SQL_INDEX_UNIQUE && fUnique != SQL_INDEX_ALL) {
      AddError(0, 0, myHstmt, "S1100", 0, "Uniqueness option type out of range");
      Return = SQL_ERROR;
   } else if (fAccuracy != SQL_ENSURE && fAccuracy != SQL_QUICK) {
      AddError(0, 0, myHstmt, "S1101", 0, "Accuracy option type out of range");
      Return = SQL_ERROR;
   } else {
       DelError(0,0,myHstmt);
      argv[0] = &Return;
      argv[1] = &myHstmt->srvHstmt;

      if (cbTableQualifier > 0) {
         argv[2] = new char[cbTableQualifier + 1];
         memcpy (argv[2], szTableQualifier, cbTableQualifier);
         ((char **) argv)[2][cbTableQualifier] = '\0';
      } else
         argv[2] = szTableQualifier;
      argv[3] = &cbTableQualifier;

      if (cbTableOwner > 0) {
         argv[4] = new char[cbTableOwner + 1];
         memcpy (argv[4], szTableOwner, cbTableOwner);
         ((char **) argv)[4][cbTableOwner] = '\0';
      } else
         argv[4] = szTableOwner;
      argv[5] = &cbTableOwner;

      if (cbTableName > 0) {
         argv[6] = new char[cbTableName + 1];
         memcpy (argv[6], szTableName, cbTableName);
         ((char **) argv)[6][cbTableName] = '\0';
      } else
         argv[6] = szTableName;
      argv[7] = &cbTableName;

      argv[8] = &fUnique;
      argv[9] = &fAccuracy;

      Exception *ex = Exception_new();
      if (!Invoke_invoke (myHstmt->hdbc->call,ex,"SQLStatistics",argv)) {
         char stat[6];
         sprintf (stat,"08%03d", ex->code);
         AddError (0, 0, myHstmt, stat, ex->code, ex->description);
         Exception_log(ex);
      }
      if (argv[2] != szTableQualifier)
         delete (char*) argv[2];
      if (argv[4] != szTableOwner)
         delete (char*) argv[4];
      if (argv[6] != szTableName)
         delete (char*) argv[6];
      Exception_delete (ex);
   }

   return (RETCODE) Return;
}

//  Have DBMS set up result set of SpecialColumns.

RETCODE SQL_API SQLSpecialColumns(
    HSTMT     hstmt,
    UWORD     fColType,
    UCHAR FAR *szTableQualifier,
    SWORD     cbTableQualifier,
    UCHAR FAR *szTableOwner,
    SWORD     cbTableOwner,
    UCHAR FAR *szTableName,
    SWORD     cbTableName,
    UWORD     fScope,
    UWORD     fNullable)
{
   CORBA_Long Return = SQL_ERROR;
   ClntHstmt *myHstmt = (ClntHstmt *) hstmt;
   void *argv[11];

   if (!myHstmt || !myHstmt->IsA(ClntHstmt::Class))
      return SQL_INVALID_HANDLE;
   else if (notOk (szTableQualifier, cbTableQualifier) ||
            notOk (szTableOwner,     cbTableOwner) || 
            notOk (szTableName,      cbTableName)) {
      AddError(0, 0, myHstmt, "S1090", 0, "Invalid string or buffer length");
      Return = SQL_ERROR;
   } else {
       DelError(0,0,myHstmt);
      argv[0] = &Return;
      argv[1] = &myHstmt->srvHstmt;
      argv[2] = &fColType;

      if (cbTableQualifier > 0) {
         argv[3] = new char[cbTableQualifier + 1];
         memcpy (argv[3], szTableQualifier, cbTableQualifier);
         ((char **) argv)[3][cbTableQualifier] = '\0';
      } else
         argv[3] = szTableQualifier;
      argv[4] = &cbTableQualifier;

      if (cbTableOwner > 0) {
         argv[5] = new char[cbTableOwner + 1];
         memcpy (argv[5], szTableOwner, cbTableOwner);
         ((char **) argv)[5][cbTableOwner] = '\0';
      } else
         argv[5] = szTableOwner;
      argv[6] = &cbTableOwner;

      if (cbTableName > 0) {
         argv[7] = new char[cbTableName + 1];
         memcpy (argv[7], szTableName, cbTableName);
         ((char **) argv)[7][cbTableName] = '\0';
      } else
         argv[7] = szTableName;
      argv[8] = &cbTableName;

      argv[9] = &fScope;
      argv[10] = &fNullable;

      Exception *ex = Exception_new();
      if (!Invoke_invoke (myHstmt->hdbc->call,ex,"SQLSpecialColumns",argv)) {
         char stat[6];
         sprintf (stat,"08%03d", ex->code);
         AddError (0, 0, myHstmt, stat, ex->code, ex->description);
         Exception_log(ex);
      }
      if (argv[3] != szTableQualifier)
         delete (char*) argv[3];
      if (argv[5] != szTableOwner)
         delete (char*) argv[5];
      if (argv[7] != szTableName)
         delete (char*) argv[7];
      Exception_delete (ex);
   }

   return (RETCODE) Return;
}

//  Have DBMS set up result set of TablePrivileges.
/*
RETCODE SQL_API SQLTablePrivileges(
    HSTMT     hstmt,
    UCHAR FAR *szTableQualifier,
    SWORD     cbTableQualifier,
    UCHAR FAR *szTableOwner,
    SWORD     cbTableOwner,
    UCHAR FAR *szTableName,
    SWORD     cbTableName)
{
   CORBA_Long Return = SQL_SUCCESS;
   if (Trace.isSet)
      Trace.Write ("SQLTablePrivileges(0x%lx,'%s',%d,'%s',%d,'%s',%d)=%d\n",
                      hstmt,
                      szTableQualifier,cbTableQualifier,
                      szTableOwner,cbTableOwner,
                      szTableName,cbTableName,
                      Return);
   return (RETCODE) Return;
}

//  Have DBMS set up result set of ColumnPrivileges.

RETCODE SQL_API SQLColumnPrivileges(
    HSTMT     hstmt,
    UCHAR FAR *szTableQualifier,
    SWORD     cbTableQualifier,
    UCHAR FAR *szTableOwner,
    SWORD     cbTableOwner,
    UCHAR FAR *szTableName,
    SWORD     cbTableName,
    UCHAR FAR *szColumnName,
    SWORD     cbColumnName)
{
   CORBA_Long Return = SQL_SUCCESS;
   if (Trace.isSet)
      Trace.Write ("SQLColumnPrivileges(0x%lx,'%s',%d,'%s',%d,'%s',%d,'%s',%d)=%d\n",
                      hstmt,
                      szTableQualifier,cbTableQualifier,
                      szTableOwner,cbTableOwner,
                      szTableName,cbTableName,
                      szColumnName,cbColumnName,
                      Return);
}

//  Have DBMS set up result set of PrimaryKeys.

RETCODE SQL_API SQLPrimaryKeys(
    HSTMT     hstmt,
    UCHAR FAR *szTableQualifier,
    SWORD     cbTableQualifier,
    UCHAR FAR *szTableOwner,
    SWORD     cbTableOwner,
    UCHAR FAR *szTableName,
    SWORD     cbTableName)
{
   CORBA_Long Return = SQL_SUCCESS;
   if (Trace.isSet)
      Trace.Write ("SQLPrimaryKeys(0x%lx,'%s',%d,'%s',%d,'%s',%d)=%d\n",
                      hstmt,
                      szTableQualifier,cbTableQualifier,
                      szTableOwner,cbTableOwner,
                      szTableName,cbTableName,
                      Return);
   return (RETCODE) Return;
}

//  Have DBMS set up result set of ForeignKeys.

RETCODE SQL_API SQLForeignKeys(
    HSTMT     hstmt,
    UCHAR FAR *szPkTableQualifier,
    SWORD     cbPkTableQualifier,
    UCHAR FAR *szPkTableOwner,
    SWORD     cbPkTableOwner,
    UCHAR FAR *szPkTableName,
    SWORD     cbPkTableName,
    UCHAR FAR *szFkTableQualifier,
    SWORD     cbFkTableQualifier,
    UCHAR FAR *szFkTableOwner,
    SWORD     cbFkTableOwner,
    UCHAR FAR *szFkTableName,
    SWORD     cbFkTableName)
{
   CORBA_Long Return = SQL_SUCCESS;
   if (Trace.isSet)
      Trace.Write ("SQLForeignKeys(0x%lx,'%s',%d,'%s',%d,'%s',%d,'%s',%d,'%s',%d,'%s',%d)=%d\n",
                      hstmt,
                      szPkTableQualifier,cbPkTableQualifier,
                      szPkTableOwner,cbPkTableOwner,
                      szPkTableName,cbPkTableName,
                      szFkTableQualifier,cbFkTableQualifier,
                      szFkTableOwner,cbFkTableOwner,
                      szFkTableName,cbFkTableName,
                      Return);
   return (RETCODE) Return;
}

//  Have DBMS set up result set of Procedures.

RETCODE SQL_API SQLProcedures(
    HSTMT     hstmt,
    UCHAR FAR *szProcQualifier,
    SWORD     cbProcQualifier,
    UCHAR FAR *szProcOwner,
    SWORD     cbProcOwner,
    UCHAR FAR *szProcName,
    SWORD     cbProcName)
{
   CORBA_Long Return = SQL_SUCCESS;
   if (Trace.isSet)
      Trace.Write ("SQLProcedures(0x%lx,'%s',%d,'%s',%d,'%s',%d)=%d\n",
                      hstmt,
                      szProcQualifier,cbProcQualifier,
                      szProcOwner,cbProcOwner,
                      szProcName,cbProcName,
                      Return);
   return (RETCODE) Return;
}

//  Have DBMS set up result set of ProcedureColumns.

RETCODE SQL_API SQLProcedureColumns(
    HSTMT     hstmt,
    UCHAR FAR *szProcQualifier,
    SWORD     cbProcQualifier,
    UCHAR FAR *szProcOwner,
    SWORD     cbProcOwner,
    UCHAR FAR *szProcName,
    SWORD     cbProcName,
    UCHAR FAR *szColumnName,
    SWORD     cbColumnName)
{
   CORBA_Long Return = SQL_SUCCESS;
   if (Trace.isSet)
      Trace.Write ("SQLProcedureColumns(0x%lx,'%s',%d,'%s',%d,'%s',%d,'%s',%d)=%d\n",
                      hstmt,
                      szProcQualifier,cbProcQualifier,
                      szProcOwner,cbProcOwner,
                      szProcName,cbProcName,
                      szColumnName,cbColumnName,
                      Return);
   return (RETCODE) Return;
}
*/
};
